Fake Data Generator for PostgreSQL
===============================================================================

`postgresql_faker` is a PostgreSQL extension based on the awesome
[Python Faker Library]. This is useful to generate random-but-meaningful
datasets for functionnal testing, anonymization, training data, etc...

This extension is simply a wrapper written in [pl/python] procedural language.

[Python Faker Library]: http://faker.rtfd.org/
[pl/python]: https://www.postgresql.org/docs/current/plpython.html

Demo
-------------------------------------------------------------------------------

You create and init the extension like this:

```sql
CREATE SCHEMA faker;

CREATE EXTENSION faker SCHEMA faker CASCADE;

SELECT faker.faker();

SELECT faker.name();
     name
---------------
 Edith Navarro
(1 row)
```

Then you can populate easily a table like this:

```sql
CREATE TABLE users (
  id BIGINT,
  firstname TEXT,
  lastname TEXT,
  email TEXT,
  birth DATE,
  ssn TEXT
);

INSERT INTO users AS
SELECT
  faker.ean()::BIGINT,
  faker.first_name(),
  faker.last_name(),
  faker.company_email(),
  faker.date_this_century()::DATE - INTERVAL '30 year',
  faker.ssn()
FROM generate_series(1,1000);
```


Providers
-------------------------------------------------------------------------------

The faker extension offers 190+ functions such `name()`, `address()`, etc.
The methods are regrouped in various providers

* [Standard Providers] are generic and covers most of your needs
* [Localized Providers] produces language-specific data (see [Localization])
* [Community Providers] are external packages that you can install with `pip`


[Standard Providers]: https://faker.readthedocs.io/en/master/providers.html
[Community Providers]: https://faker.readthedocs.io/en/master/communityproviders.html
[Localized Providers]: https://faker.readthedocs.io/en/master/locales.html

Install
-------------------------------------------------------------------------------

You need to install python3, pl/python and the latest version of the Faker
library. Here's an exemple using debian/ubuntu and PostgreSQL 12:

```sql
apt install python3-pip postgresql-plpython3-12 pgxnclient
pip install faker
pgxn install postgresql_faker
```


Localization
-------------------------------------------------------------------------------

faker.faker() can take a locale as an argument, to return localized data.
If no localized provider is found, the fallback si the `en_US` locale.

```sql
SELECT faker.faker('it_IT');
SELECT faker.name();
     name
---------------------------
   Sig. Alighieri Monti
```

faker.faker() also supports multiple locales.

```sql
SELECT faker.faker(ARRAY['it_IT', 'en_US', 'ja_JP']);
```

For the complete list of available locale, please read the Faker doc:

https://faker.readthedocs.io/en/master/locales.html

Seeding The Generator
-------------------------------------------------------------------------------

When using Faker for unit testing, you will often want to generate the same
data set. For convenience, the extension provides a `seed()` function,
which seeds the shared random number generator. Calling the same methods
with the same version of faker and seed produces the same results.

```sql
SELECT faker.faker();

SELECT faker.seed(4321);
SELECT faker.name() FROM generate_series(1,3);
    name
-------------
 Jason Brown
 Jacob Stein
 Cody Brown
(3 rows)

SELECT faker.seed(4321);
SELECT faker.name() FROM generate_series(1,3);
    name
-------------
 Jason Brown
 Jacob Stein
 Cody Brown
(3 rows)
```

Limitations
-------------------------------------------------------------------------------

### Casting

All function will return `TEXT` values. You need to convert the result into the
data type you want:

```sql
# SELECT faker.year() = 2020;
ERROR:  operator does not exist: text = integer

# SELECT faker.year()::INT = 2020;
 ?column?
----------
 f
(1 row)
```

### Reserved keywords

The python Faker methods `time()` and `date()` are not available because their
name are reserved PostgreSQL keywords.

* instead of `faker.date()`, you can use `faker.date_between()`
* instead of `faker.time()`, you can use `(random() * interval '1d')::TIME`

Licence
-------------------------------------------------------------------------------

`postgresql_faker` is released under the PostgreSQL License.


